///////////////////////////////////////////////////////////////////////////
// Copyright and License Notice
///////////////////////////////////////////////////////////////////////////
// These files contain code made by Will Chang under supervision of
// Matthias Zwicker for his Ph.D. work with the University of
// California, San Diego.
//   
// Copyright (c) 2009 - 2011, William Y. Chang
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
//     * Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//     
//     * Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the following
//     disclaimer in the documentation and/or other materials provided
//     with the distribution.
// 
//     * Neither the name of University of California, San Diego nor
//     the names of its contributors may be used to endorse or promote
//     products derived from this software without specific prior
//     written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
///////////////////////////////////////////////////////////////////////////

#include "xmlParam.h"
#include <iostream>
#include <QFile>
#include <QTextStream>
#include <QStringList>

using namespace std;

xmlParam::xmlParam() : topName("libXMLParam") {
	// nothing yet!
}

xmlParam::xmlParam( char *name ) : topName(name) {
	// nothing yet!
}

string xmlParam::getTopName() {
	return topName;
}

/**
* Populate the vectors that hold the names and references of the parameters.
* This method populates paramList (names in XML file), paramTypes (type of
* the parameter variables), and paramPtr (a pointer (casted to void*) to the
* actual variable).
*/
void xmlParam::addParam(string name, string type, void* ptr, const char *comment/* = NULL*/) {
	paramList.push_back(name);
	paramTypes.push_back(type);
	paramPtr.push_back(ptr);
	if (comment == NULL) {
		paramComment.push_back(string(""));
	}
	else {
		paramComment.push_back(string(comment));
	}
}


/**
* Set the global parameters to default values.
*/
void xmlParam::resetParams() {
	for (unsigned int i = 0; i < paramList.size(); i++) {
		if (paramTypes[i] == "string") {
			*((string*)paramPtr[i]) = string("");
		}
		else if (paramTypes[i] == "bool") {
			*((bool*)paramPtr[i]) = false;
		}
		else if (paramTypes[i] == "float") {
			*((float*)paramPtr[i]) = -1.0f;
		}
		else if (paramTypes[i] == "double") {
			*((double*)paramPtr[i]) = -1.0;
		}
		else if (paramTypes[i] == "int") {
			*((int*)paramPtr[i]) = -1;
		}
		else if (paramTypes[i] == "stringArray") {
			((StringArray*)paramPtr[i])->clear();
		}
		else if (paramTypes[i] == "boolArray") {
			((BoolArray*)paramPtr[i])->clear();
		}
		else if (paramTypes[i] == "floatArray") {
			((FloatArray*)paramPtr[i])->clear();
		}
		else if (paramTypes[i] == "intArray") {
			((IntArray*)paramPtr[i])->clear();
		}
	}
}


/**
* Print the list of parameters to stdout using printf.
*/
void xmlParam::printParams() {
	for (unsigned int i=0; i < paramList.size(); i++) {
		printf("Parameter [%s]: ", paramList[i].c_str());
		if (paramTypes[i] == "string") {
			cout << "[" << ((string*)paramPtr[i])->c_str() << "]" << endl;
		}
		else if (paramTypes[i] == "bool") {
			cout << "[" << (*((bool*)paramPtr[i])?"true":"false") << "]" << endl;
		}
		else if (paramTypes[i] == "float") {
			cout << "[" << (*((float*)paramPtr[i])) << "]" << endl;
		}
		else if (paramTypes[i] == "double") {
			cout << "[" << (*((double*)paramPtr[i])) << "]" << endl;
		}
		else if (paramTypes[i] == "int") {
			cout << "[" << (*((int*)paramPtr[i])) << "]" << endl;
		}
		else if (paramTypes[i] == "stringArray") {
			StringArray &val(*((StringArray*)paramPtr[i]));

			cout << "[";
			for (unsigned int j=0; j < val.size(); j++) {
				if (j > 0) cout << ",";
				cout << "\"" << val[j].c_str() << "\"";
			}
			cout << "]" << endl;
		}
		else if (paramTypes[i] == "boolArray") {
			BoolArray &val(*((BoolArray*)paramPtr[i]));

			cout << "[";
			for (unsigned int j=0; j < val.size(); j++) {
				if (j > 0) cout << ",";
				cout << val[j];
			}
			cout << "]" << endl;
		}
		else if (paramTypes[i] == "floatArray") {
			FloatArray &val(*((FloatArray*)paramPtr[i]));

			cout << "[";
			for (unsigned int j=0; j < val.size(); j++) {
				if (j > 0) cout << ",";
				cout << val[j];
			}
			cout << "]" << endl;
		}
		else if (paramTypes[i] == "intArray") {
			IntArray &val(*((IntArray*)paramPtr[i]));

			cout << "[";
			for (unsigned int j=0; j < val.size(); j++) {
				if (j > 0) cout << ",";
				cout << val[j];
			}
			cout << "]" << endl;
		}
	}
}


/**
* Print the list of parameters to stdout using printf.
*/
void xmlParam::printParamsINI() {
	for (unsigned int i=0; i < paramList.size(); i++) {
		printf("%s", paramList[i].c_str());
		if (paramTypes[i] == "string") {
			cout << " = \"" << ((string*)paramPtr[i])->c_str() << "\"" << endl;
		}
		else if (paramTypes[i] == "bool") {
			cout << " = " << (*((bool*)paramPtr[i])?"true":"false") << endl;
		}
		else if (paramTypes[i] == "float") {
			cout << " = " << *((float*)paramPtr[i]) << endl;
		}
		else if (paramTypes[i] == "double") {
			cout << " = " << *((double*)paramPtr[i]) << endl;
		}
		else if (paramTypes[i] == "int") {
			cout << " = " << *((int*)paramPtr[i]) << endl;
		}
		else if (paramTypes[i] == "stringArray") {
			StringArray &val(*((StringArray*)paramPtr[i]));

			cout << "TypeFA = \"";
			for (unsigned int j=0; j < val.size(); j++) {
				if (j > 0) cout << ",";
				cout << "\"" << val[j].c_str() << "\"";
			}
			cout << "\"" << endl;
		}
		else if (paramTypes[i] == "boolArray") {
			BoolArray &val(*((BoolArray*)paramPtr[i]));

			cout << "TypeFA = \"";
			for (unsigned int j=0; j < val.size(); j++) {
				if (j > 0) cout << ",";
				cout << val[j];
			}
			cout << "\"" << endl;
		}
		else if (paramTypes[i] == "floatArray") {
			FloatArray &val(*((FloatArray*)paramPtr[i]));

			cout << "TypeFA = \"";
			for (unsigned int j=0; j < val.size(); j++) {
				if (j > 0) cout << ",";
				cout << val[j];
			}
			cout << "\"" << endl;
		}
		else if (paramTypes[i] == "intArray") {
			IntArray &val(*((IntArray*)paramPtr[i]));

			cout << "TypeFA = \"";
			for (unsigned int j=0; j < val.size(); j++) {
				if (j > 0) cout << ",";
				cout << val[j];
			}
			cout << "\"" << endl;
		}
	}
}


/**
* Create an XML document that writes the list of parameters and their values.
*/
QDomDocument xmlParam::makeParamXML() {
	QDomDocument doc(QString(topName.c_str()));
	QDomElement root = doc.createElement(QString(topName.c_str()));
	doc.appendChild(root);

	for (unsigned int i=0; i < paramList.size(); i++) {
		QDomElement tag = doc.createElement(paramList[i].c_str());
		tag.setAttribute(QString("type"), QString(paramTypes[i].c_str()));
		root.appendChild(tag);

		QString str("");
		if (paramTypes[i] == "string") {
			str = QString(((string*)paramPtr[i])->c_str());
		}
		else if (paramTypes[i] == "bool") {
			str = QString::number(*((bool*)paramPtr[i]));
		}
		else if (paramTypes[i] == "float") {
			str = QString::number(*((float*)paramPtr[i]));
		}
		else if (paramTypes[i] == "double") {
			str = QString::number(*((double*)paramPtr[i]));
		}
		else if (paramTypes[i] == "int") {
			str = QString::number(*((int*)paramPtr[i]));
		}
		else if (paramTypes[i] == "stringArray") {
			StringArray &val(*((StringArray*)paramPtr[i]));

			for (unsigned int j=0; j < val.size(); j++) {
				if (j > 0) str = str + QString(",");
				str = str + QString("%1,").arg(val[j].c_str());
			}
		}
		else if (paramTypes[i] == "boolArray") {
			BoolArray &val(*((BoolArray*)paramPtr[i]));

			for (unsigned int j=0; j < val.size(); j++) {
				if (j > 0) str = str + QString(",");
				str = str + QString("%1,").arg(val[j]);
			}
		}
		else if (paramTypes[i] == "floatArray") {
			FloatArray &val(*((FloatArray*)paramPtr[i]));

			for (unsigned int j=0; j < val.size(); j++) {
				if (j > 0) str = str + QString(",");
				str = str + QString("%1,").arg(val[j]);
			}
		}
		else if (paramTypes[i] == "intArray") {
			IntArray &val(*((IntArray*)paramPtr[i]));

			for (unsigned int j=0; j < val.size(); j++) {
				if (j > 0) str = str + QString(",");
				str = str + QString("%1,").arg(val[j]);
			}
		}
		tag.appendChild(doc.createTextNode(str));

		if (paramComment[i] != "") {
			// add the comment
			tag.appendChild(doc.createComment(QString(paramComment[i].c_str())));
		}
	}

	return doc;
}


/**
* Write an XML document to a file.
*/
bool xmlParam::writeParamXML(const char* filename, QDomDocument &doc) {

	// open the file
	QFile file(filename);
	if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
		cout << "Could not open file " << filename << " for writing!" << endl;
		return false;
	}

	QTextStream out(&file);
	out << doc.toString() << endl;

	file.close();
	return true;
}

/**
* Read an XML document from a file.
*/
bool xmlParam::readParamXML(const char* filename, QDomDocument &doc) {
	QFile file(filename);

	// we should really make this a class and throw exceptions here
	if (!file.open(QIODevice::ReadOnly)) {
		cout << "Could not open file " << filename << "!" << endl;
		return false;
	}

	if (!doc.setContent(&file)) {
		file.close();
		cout << "Could not parse XML file " << filename << "!" << endl;
		return false;
	}
	file.close();

	return true;
}

/**
* Populate the parameter values from the values written in an XML document.
*/
void xmlParam::loadParamXML(QDomDocument &doc) {
	// populate the values
	for (unsigned int i=0; i < paramList.size(); i++) {
		// find the tag with the parameter name
		QDomNodeList list = doc.elementsByTagName(QString(paramList[i].c_str()));

		if (list.count() == 1) {
			// retrieve the text node value
			QDomElement elt = list.item(0).toElement();
			QString content = elt.text();

			//cout << qPrintable(elt.tagName()) << " --> " << qPrintable(content) << endl;

			if (paramTypes[i] == "string") {
				*((string*)paramPtr[i]) = string(qPrintable(content));
			}
			else if (paramTypes[i] == "bool") {
				*((bool*)paramPtr[i]) = content.toInt();
			}
			else if (paramTypes[i] == "float") {
				*((float*)paramPtr[i]) = content.toFloat();
			}
			else if (paramTypes[i] == "double") {
				*((double*)paramPtr[i]) = content.toDouble();
			}
			else if (paramTypes[i] == "int") {
				*((int*)paramPtr[i]) = content.toInt();
			}
			else if (paramTypes[i] == "stringArray") {
				StringArray &val(*((StringArray*)paramPtr[i]));
				val.clear();

				QStringList contentList = content.split(",", QString::SkipEmptyParts);

				for (unsigned int j = 0; j < contentList.size(); j++) {
					val.push_back(string(qPrintable(contentList[j])));
				}
			}
			else if (paramTypes[i] == "boolArray") {
				BoolArray &val(*((BoolArray*)paramPtr[i]));
				val.clear();

				QStringList contentList = content.split(",", QString::SkipEmptyParts);

				for (unsigned int j = 0; j < contentList.size(); j++) {
					val.push_back(contentList[j].toInt());
				}
			}
			else if (paramTypes[i] == "floatArray") {
				FloatArray &val(*((FloatArray*)paramPtr[i]));
				val.clear();

				QStringList contentList = content.split(",", QString::SkipEmptyParts);

				for (unsigned int j = 0; j < contentList.size(); j++) {
					val.push_back(contentList[j].toFloat());
				}
			}
			else if (paramTypes[i] == "intArray") {
				IntArray &val(*((IntArray*)paramPtr[i]));
				val.clear();

				QStringList contentList = content.split(",", QString::SkipEmptyParts);

				for (unsigned int j = 0; j < contentList.size(); j++) {
					val.push_back(contentList[j].toInt());
				}
			}
		}
		else {
			cout << "Could not find element " << paramList[i].c_str() << "!" << endl;
		}
	}
}

bool xmlParam::initParamXML(int argc, char *argv[]) {
	
	char input = 'y';
	bool prompt = true;
	char *inputFile = argv[1];

	if (argc < 2) {
		printUsage(argv[0]);
		return false;
	}

	// Load input parameter file
	QDomDocument doc(QString(this->getTopName().c_str()));
	setXMLParamList(*this);

	// writing mode -- always exit with false so that the program can exit
	if (string(argv[1]) == "-w") {
		if (argc == 3) {
			if (!this->writeParamXML(argv[2], this->makeParamXML())) {
				cerr << "Could not write parameters to " << argv[2] << "." << endl;
			}
			else {
				cout << "Wrote parameters to " << argv[2] << "." << endl;
			}
		}
		else if (argc > 3) {
			printUsage(argv[0]);
		}

		return false;
	}
	else if (string(argv[1]) == "-b") {
		// Turn on batch mode: don't prompt to continue processing
		prompt = false;

		if (argc < 3) {
			cerr << "Need an input file to process in batch mode!" << endl;
			return false;
		}
		else {
			// use the third argument as the input file
			inputFile = argv[2];
		}
	}

	// reading mode
	if (!this->readParamXML(inputFile, doc)) {
		cerr << "Couldn't read parameters from " << inputFile << "." << endl;
	}
	this->loadParamXML(doc);
	this->printParams();

	// prompt to start processing
	if (prompt) {
		cout << endl << "Start Processing? (y/n): ";
		cin >> input;

		return ((input == 'y') || (input == 'Y'));
	}

	return true;
}

void xmlParam::printUsage(char *argv0)
{
	cout << "Usage: " << argv0 << " [options] <filename>" << endl << endl;
	cout << "Options:" << endl << "  -w  <outfile>     Write out the parameters to a sample XML file" << endl;
	cout << "  <infile>          Read parameters from input XML file" << endl;
	cout << endl;
}
