///////////////////////////////////////////////////////////////////////////
// Copyright and License Notice
///////////////////////////////////////////////////////////////////////////
// These files contain code made by Will Chang under supervision of
// Matthias Zwicker for his Ph.D. work with the University of
// California, San Diego.
//   
// Copyright (c) 2009 - 2011, William Y. Chang
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
//     * Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//     
//     * Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the following
//     disclaimer in the documentation and/or other materials provided
//     with the distribution.
// 
//     * Neither the name of University of California, San Diego nor
//     the names of its contributors may be used to endorse or promote
//     products derived from this software without specific prior
//     written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
///////////////////////////////////////////////////////////////////////////

#ifndef DEFORMEDIT_H
#define DEFORMEDIT_H

#include <QWidget>
#include "generatedfiles/ui_DeformEdit.h"
#include "ParamSelect.h"

enum DeformEditSignals {
	NONE = 0,

	// Sliders
	NUM_BONES,
	NUM_DIVISIONS,
	NUM_CORRESPONDENCES,
	MAX_CORRESP_DIST,
	MAX_BDY_CORRESP_DIST,
	DEFORM_EM_CONSTRAINT_E_WEIGHT,
	DEFORM_EM_SMOOTHNESS_E_WEIGHT,
	DEFORM_EM_CONSTRAINT_M_WEIGHT,
	DEFORM_EM_SMOOTHNESS_M_WEIGHT,
	DEFORM_EM_UNITY_M_WEIGHT,
	DEFORM_EM_DISCRETE_M_WEIGHT,
	SOLVER_MAX_ITERATIONS,
	ESTEP_MAX_ITERATIONS,
	PROJ_MAX_ITERATIONS,
	IRLS_MAX_ITERATIONS,

	// Buttons
	OUTPUT_STATS,
	SETUP_DEFORM,
	SOLVE_DEFORM_INIT,
	SOLVE_DEFORM_TRANSFORM,
	SOLVE_DEFORM_WEIGHT,
	SOLVE_DEFORM_CORRESP,
	SOLVE_DEFORM_AUTO,
	EXPORT_DEFORM_RESULT,
	INVERSE_DEFORM,
	PAUSE_SOLVER,
	STEP_SOLVER,
	STOP_SOLVER,
	ACCUMULATE_GEOMETRY,
	REQUEST_NEXT_FRAME,
	REVERT_PREVIOUS_SOLUTION,

	// Combo Boxes & Check Boxes
	OTHER_TASKS,
	SOLVE_INITIAL_RIGID,
	SOLVE_FINAL_WEIGHTS,
	UPDATE_CORRESPONDENCE_TYPE,

	// Old stuff
	GRAPH_NODES,
	GRAPH_NEIGHBORS,
	OVERLAP_FACTOR,
	MAX_ITERATIONS_SOLVER,
	MAX_ITERATIONS_PROJECTION,
	ED_WEIGHTS_ROTATION,
	ED_WEIGHTS_SMOOTH,
	ED_WEIGHTS_CONSTRAINT,
	WD_WEIGHTS_SMOOTH,
	WD_WEIGHTS_CONSTRAINT
};

class DeformEdit : public QWidget
{
	Q_OBJECT

public:
	DeformEdit(QWidget *parent = 0);
	~DeformEdit();

	void initRange( int which, double minRange, double maxRange, double initValue );
	void init_solveInitialRigidCheckBox( bool value );
	void init_solveFinalWeightsCheckBox( bool value );
	void init_correspondenceTypeCombo( int type );

private slots:
	void on_pauseSolverButton_clicked();
	void on_stepSolverButton_clicked();
	void on_solveFinalWeightsCheckBox_stateChanged(int);
	void on_RevertSolutionButton_clicked();
	void on_requestNextFrameButton_clicked();
	void on_AccumulateGeometryButton_clicked();
	void on_correspondenceTypeCombo_currentIndexChanged(int);
	void on_solveInitialRigidCheckBox_stateChanged(int);
	void on_OtherTasksComboBox_currentIndexChanged(int);
	void on_stopSolverButton_clicked();
	void on_inverseDeformButton_clicked();
	void on_exportResultButton_clicked();
	void on_solveDeformAutoButton_clicked();
	void on_solveDeformUpdateCorrespButton_clicked();
	void on_solveDeformWeightButton_clicked();
	void on_solveDeformTransformButton_clicked();
	void on_solveDeformInitButton_clicked();
	void on_setupDeformButton_clicked();
	void controlChanged(int id, double value);
	void on_outputStats_clicked();

private:
	Ui::DeformEditClass ui;
	std::vector<ParamSelect*> controls;

	void init();
	ParamSelect* findControl(int id);

signals:
	void valueChanged(int which, double value);
};

#endif // DEFORMEDIT_H
