///////////////////////////////////////////////////////////////////////////
// Copyright and License Notice
///////////////////////////////////////////////////////////////////////////
// These files contain code made by Will Chang under supervision of
// Matthias Zwicker for his Ph.D. work with the University of
// California, San Diego.
//   
// Copyright (c) 2009 - 2011, William Y. Chang
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
//     * Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//     
//     * Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the following
//     disclaimer in the documentation and/or other materials provided
//     with the distribution.
// 
//     * Neither the name of University of California, San Diego nor
//     the names of its contributors may be used to endorse or promote
//     products derived from this software without specific prior
//     written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
///////////////////////////////////////////////////////////////////////////

#ifndef __FILE_SEQUENCE_H__
#define __FILE_SEQUENCE_H__

#include <QString>
#include "types.h"

void splitPath( const QString &path, QString &dir, QString &file );		/**< Split a path to its immediate container and file. */
QString grabFile( const QString &path );
QString grabPath( const QString &path );
QString removeExt( const QString &filename );							/**< Remove the extension from the filename */

/**
 * Class for keeping a list of files.
 */
class FileSequence
{
public:
	// Various constructors
	FileSequence();
	FileSequence( QString filename );
	FileSequence( const char *filename );
	FileSequence( std::string filename );

	// Construction
	void construct( QString filename );

	// Access
	QString operator[]( index_type i ) const;
	QString operator[]( QString arg ) const;

	// Check
	bool hasNumber() const;
	bool valid() const;

	// Optional parameter storage
	int startFrame, endFrame, skipFrame;

private:

	// Data
	QString format;
	int numDigits;

};

#endif
