///////////////////////////////////////////////////////////////////////////
// Copyright and License Notice
///////////////////////////////////////////////////////////////////////////
// These files contain code made by Will Chang under supervision of
// Matthias Zwicker for his Ph.D. work with the University of
// California, San Diego.
//   
// Copyright (c) 2009 - 2011, William Y. Chang
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
//     * Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//     
//     * Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the following
//     disclaimer in the documentation and/or other materials provided
//     with the distribution.
// 
//     * Neither the name of University of California, San Diego nor
//     the names of its contributors may be used to endorse or promote
//     products derived from this software without specific prior
//     written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
///////////////////////////////////////////////////////////////////////////

#ifndef __GEOM_UTIL_H__
#define __GEOM_UTIL_H__

#include "types.h"

real_type pointBoxSqDistance ( const real3_type &p, const real3_type &boxMin, const real3_type &boxMax, real3_type &closestPoint );
real_type lowerBoundTriangleSqDistance( const real3_type &p, const real3_type &t0, const real3_type &t1, const real3_type &t2 );


// Cross product.
inline real3_type cross(const real3_type &v1, const real3_type &v2)
{
	return real3_type(v1.y()*v2.z() - v1.z()*v2.y(), v1.z()*v2.x() - v1.x()*v2.z(), v1.x()*v2.y() - v1.y()*v2.x());
}


// Inverting trilinear interpolation (adapted from hexahedronTest project)
void invert(Geometry::Matrix33f &mat);
void bilinearInvert3D(const real3_type &p, const real3_type &v0, const real3_type &v1, const real3_type &v2, const real3_type &v3, real_type &alpha, real_type &beta);
real3_type trilinearNewtonUpdate( const real3_type &p, real3_type **verts, const real3_type &weights, real3_type &f );
real3_type invertTrilinear(const real3_type &p, real3_type **verts, int maxItr = 10);
bool hexahedronTestInside( const real3_type &p, real3_type **verts );

// Creating list of edges for a mesh
class Mesh3_real;
void createEdgeList( const Mesh3_real &input, EdgeList &edgeList, AdjList &adjList );

#endif
